// (c) 2015, SAS Institute Inc.

/*jslint browser: true, plusplus: true, devel: true, regexp: true */
/*global jQuery: false, SASEpisodeManager: false, edm: false, console: false */
/* eslint no-console:0 */

//Defect S1208483 Hang on Opening Copied Episode Definition hotfix branch: f0sea31
 
jQuery.sap.declare("edm.utils.DataTransmogrifier");




jQuery.sap.require(
    "sap.m.MessageBox",
    "sap.m.MessageToast"
);


edm.utils.DataTransmogrifier = {



    "epidefFromMidtierToUIFormat": function (results) {
        "use strict";
        var config = SASEpisodeManager.config;




        //we found a late bug in SEA 3.1...nobody actually did a sort of the trigger rules by their
        //RULE_COMPONENT_ORDER_NO, and that never appeared as a column in the gui, so the order we showed
        //the triggers in EDM was undetermined!  So before the gui even gets a definition, I'll do the sort
        //now so the individual triggers are displayed in their model-order
        function sortByRuleNumbers(data) {
            "use strict";

            //note that we are assuming that any RuleComponents entered by carolyn's tool will never skip RULE_COMPONENT_NOs (ie
            //she'd never make a rule with compNos 2 and 3 and skip 0 and 1...that would break all the current trigger insertion code.
            if(data && data.EpisodeRule && data.EpisodeRule.RuleComponents && data.EpisodeRule.RuleComponents.length > 1) {
                data.EpisodeRule.RuleComponents.sort(function (a, b) {
                    return a.RULE_COMPONENT_NO - b.RULE_COMPONENT_NO;
                });
            }
            return data;
        }

        //I don't want to render all of a certain type of code in just one long table....too many!  Instead
        //i'll chop them up by type for better rendering and operate on THOSE lists, but when I persist the
        //definition, i'll need to recreate the base codelist from all those pieces.
        function explodeCodes(rawForm) {
            var triggerCodes = {},
                typicalCodes = {},
                complicationCodes = {},
                exclusionCodes = {},
                rawCodes = rawForm.Codes,
                codeTypes = SASEpisodeManager.lookups.getCodeTypes(),
                totalComplicationsOnLoad = 0,
                totalTypicalsOnLoad = 0,
                totalExclusionsOnLoad = 0,
                totalTriggersOnLoad = 0,
                complicationsByTypePointer,
                typicalsByTypePointer,
                triggersByTypePointer,
                exclusionsByTypePointer;

            if(SASEpisodeManager.logger.isLoggable(jQuery.sap.log.Level.TRACE)) {
                console.log("rawForm follows:");
                console.dir(rawForm);
            }
            //make empty HASHES for each type of code for each of triggers, typicals, complications and exclusions (so
            // I can bind a table to each)
            codeTypes.forEach(function (aType) {
                triggerCodes[aType[config.CodeTypes_code_attributeName]] = {};
                typicalCodes[aType[config.CodeTypes_code_attributeName]] = {};
                complicationCodes[aType[config.CodeTypes_code_attributeName]] = {};
                exclusionCodes[aType[config.CodeTypes_code_attributeName]] = {};
            });

            function convertHashesToArrays(hashOfHashes){
                //Now I need to convert passed hashOfHashes into a hashOfArrays so that I can bind
                //uiTables to each of the subarrays (you can't bind to a hash in openui5)
                var codeTypesInSystem = SASEpisodeManager.lookups.getCodeTypes(),
                    hashOfArrays = {};

                codeTypesInSystem.forEach(function (codeType) {
                    var typeHash = hashOfHashes[codeType[config.CodeTypes_code_attributeName]],
                        typeArray = [],
                        keys;
                    if(typeHash) {
                        keys  = Object.keys(typeHash);
                        keys.forEach(function (key) {
                            typeArray.push(typeHash[key]);
                        });
                    }
                    hashOfArrays[codeType[config.CodeTypes_code_attributeName]] = typeArray;
                });
                return hashOfArrays;
            }

            //I've got a code row that either needs to be added to the array, or if it is already present, i need to merge this one
            //into the existing one.  Return a 1 if you added a new row to the array
            function pushOrMerge (typeHash, codeRow) {
                var retCount = 0,  //return 0 if you did a merge, 1 if you added a row
                    foundHimRow,
                    compositeKey = codeRow[config.episode_code_set_attributeName] + ":" + codeRow[config.episode_code_value_attributeName];

                foundHimRow = typeHash[compositeKey];
                if(foundHimRow){
                    //gotta merge these two rows
                    //And why are there duplicates???
                    //the duplicates are because of how code_category was modeled, and the only thing I care about from that
                    //is that I want the CODE_CATEGORY_NM from the BETOS category, not any other....so
                    if(codeRow[config.episode_code_category_name_attributeName] === "BETOS"){
                        foundHimRow[config.episode_code_group_attributeName]
                            = codeRow[config.episode_code_group_attributeName];
                        foundHimRow[config.episode_code_category_name_attributeName]
                            = codeRow[config.episode_code_category_name_attributeName];
                    }
                } else {

                    //right now the only value in episode_code_category_name_attributeName i want to see is if it is a BETOS, so i'll
                    //blank out the value if it is any other type
                    if(codeRow[config.episode_code_category_name_attributeName] !== "BETOS") {
                        codeRow[config.episode_code_group_attributeName] = "";
                    }

                    typeHash[compositeKey] = codeRow;
                    retCount = 1;
                }
                return retCount;
            }


            //now divvy up all the codes in this definition according to where they belong (and they can be in multiple arrays)
            rawCodes.forEach(function (code) {
                if (code[config.episode_code_complication_attributeName] === "Y") {
                    complicationsByTypePointer = complicationCodes[code[config.episode_code_type_attributeName]];
                    totalComplicationsOnLoad += pushOrMerge(complicationsByTypePointer, code);
                }
                if (code[config.episode_code_typical_attributeName] === "Y") {
                    typicalsByTypePointer = typicalCodes[code[config.episode_code_type_attributeName]];
                    totalTypicalsOnLoad += pushOrMerge(typicalsByTypePointer, code);
                }
                if (code[config.episode_code_trigger_attributeName] === "Y") {
                    triggersByTypePointer = triggerCodes[code[config.episode_code_type_attributeName]];
                    totalTriggersOnLoad += pushOrMerge(triggersByTypePointer, code);
                }
                if ((code[config.episode_code_excludeOn_IP_flag_attributeName] === "Y")
                    || (code[config.episode_code_excludeOn_OP_flag_attributeName] === "Y")
                    || (code[config.episode_code_excludeOn_PB_flag_attributeName] === "Y")
                    || (code[config.episode_code_excludeOn_RX_flag_attributeName] === "Y")
                    || (code[config.episode_code_excludeOn_ALL_flag_attributeName] === "Y")
                ) {

                    //if all claim types flag is set, make sure the individual values are also set because those
                    //are the guys i'm going to render
                    if(code[config.episode_code_excludeOn_ALL_flag_attributeName] === "Y"){
                        code[config.episode_code_excludeOn_OP_flag_attributeName] = "Y";
                        code[config.episode_code_excludeOn_PB_flag_attributeName] = "Y";
                        code[config.episode_code_excludeOn_RX_flag_attributeName] = "Y";
                        code[config.episode_code_excludeOn_ALL_flag_attributeName] = "Y";
                    }

                    exclusionsByTypePointer = exclusionCodes[code[config.episode_code_type_attributeName]];
                    totalExclusionsOnLoad += pushOrMerge(exclusionsByTypePointer, code);
                }
            });
            rawForm.triggerCodes = convertHashesToArrays(triggerCodes);
            rawForm.typicalCodes = convertHashesToArrays(typicalCodes);
            rawForm.exclusionCodes = convertHashesToArrays(exclusionCodes);
            rawForm.complicationCodes = convertHashesToArrays(complicationCodes);
            rawForm.totalComplicationsOnLoad = totalComplicationsOnLoad;
            rawForm.totalTypicalsOnLoad = totalTypicalsOnLoad;
            rawForm.totalTriggersOnLoad = totalTriggersOnLoad;
            rawForm.totalExclusionsOnLoad = totalExclusionsOnLoad;
            return rawForm;
        }

        results = explodeCodes(results);
        results = sortByRuleNumbers(results);
        return results;
    },

    "initializeCodeRow": function (newRow) {
        "use strict";
        var config = SASEpisodeManager.config;

        if(newRow.CODE_VALUE === "") {  //the all code is supposed to be a blank, not an empty string...
            newRow.CODE_VALUE = " ";
        }

        if(!newRow.hasOwnProperty(config.episode_code_sufficient_attributeName)){
            newRow[config.episode_code_sufficient_attributeName] = "N";
        }
        if(!newRow.hasOwnProperty(config.episode_code_use_all_levels_attributeName)){
            newRow[config.episode_code_use_all_levels_attributeName] = "N";
        }
        if(!newRow.hasOwnProperty(config.episode_code_complicationType_code_attributeName)){
            newRow[config.episode_code_complicationType_code_attributeName] = null;
        }
        if(!newRow.hasOwnProperty(config.episode_code_pas_attributeName)){
            newRow[config.episode_code_pas_attributeName] = "N";
        }
        if(!newRow.hasOwnProperty(config.episode_code_core_attributeName)){
            newRow[config.episode_code_core_attributeName] = "N";
        }
        if(!newRow.hasOwnProperty(config.episode_code_qualifying_attributeName)){
            newRow[config.episode_code_qualifying_attributeName] = "N";
        }

    },


    "initializeCodeRowForSubType": function (newRow) {
        "use strict";
        var config = SASEpisodeManager.config;

        if(!newRow.hasOwnProperty(config.episode_subtype_group_name_attributeName)){
            newRow[config.episode_subtype_group_name_attributeName] = "SG00000";
        }
        if(!newRow.hasOwnProperty(config.episode_subtype_group_description_attributeName)){
            newRow[config.episode_subtype_group_description_attributeName] = "Undefined group";
        }

    },



    "epidefFromUIToMidtierFormat": function (midTierFormOfModel) {
        "use strict";
        var config = SASEpisodeManager.config;

        //the 31 datamodel just uses a single flat list of codes with little attribute flags on each for the different
        //uses, which SUCKS for the ui so the ui form has them split by use and then by type.  I need to morph
        //that form back into the simple datamodel structure here
        function implodeCodes(internalForm) {
            var triggerCodes = internalForm.triggerCodes,
                typicalCodes = internalForm.typicalCodes,
                exclusionCodes = internalForm.exclusionCodes,
                complicationCodes = internalForm.complicationCodes,
                typicalHash = {},

                typicalTypeList,
                triggerTypeList,
                complicationTypeList,
                exclusionTypeList,

                newCodesList = [],
                codeTypes = SASEpisodeManager.lookups.getCodeTypes(),
                codeSets = SASEpisodeManager.lookups.getCodeSetsInfo(),
                codesetHash;


            if(SASEpisodeManager.logger.isLoggable(jQuery.sap.log.Level.TRACE)) {
                console.log("internalForm follows:");
                console.dir(internalForm);
            }

            //ok, so the internal form is a tree with typical, trigger, complication and exclusions all s separated, and then in each of
            //those the different code TYPES (DX vs PX vs whatever) separated in sub-arrays.

            //so first off, we use arrays to facilitate binding, but now that we're about to do some merging a hash
            //would be MUCH faster and easier.
            //And since typicals usually have the most codes already, we'll fold the other three
            //trees into that tree
            //
            // SO let's generate hashes of each of the code-type-specific arrays of typicals to work with instead.
            //
            // BUT, remember that we can have multiple codes of the same type and same code value, so I need to wrap all of this
            // with what codeset this code is from (which is very wasteful, but most of these will be empty so no biggie.
            // So first off, let's make a tree of hashes based on codeset, then by type


            codeSets.forEach(function (aCodeset) {
                //make the root of all codes for this codeset in the hash
                codesetHash = {};
                typicalHash["" + aCodeset.id] = codesetHash;
                codeTypes.forEach(function (aType) {
                    var minihash = {};

                    typicalTypeList = typicalCodes[aType[config.CodeTypes_code_attributeName]];
                    //if this is a made-from scratch epidef, the sub-arrays might not even be there so check!
                    if(typicalTypeList && typicalTypeList.length) {
                        typicalTypeList.forEach(function (row) {
                                if(row[config.episode_code_set_attributeName] === aCodeset.id) {
                                    //we are doing two things at once here...we are hashing our typical codes, but we
                                    //also primed this with data that had typical, complication, triggering etc all mixed together so
                                    //we need to reset that to blank and then merge in the CURRENT values.
                                    // At this point, I don't know if the code rows in typical or complication or whatever have
                                    // diverged or not, so I need to assume that they have and merge them together as I encounter them.
                                    // So for these codes that are typical, we can ONLY count on their typical-ity
                                    row[config.episode_code_typical_attributeName] = "Y";
                                    row[config.episode_code_complication_attributeName] = "N";
                                    row[config.episode_code_trigger_attributeName] = "N";
                                    row[config.episode_code_exclusionType_code_attributeName] = "N";
                                    minihash[row[config.episode_code_value_attributeName]] = row;
                                }
                            }
                        );
                    }
                    codesetHash[aType[config.CodeTypes_code_attributeName]] = minihash;
                });
            });



            //ok, so now we have a hash of hashes of typicals to work with...now we can merge the other three code classes into him

            codeSets.forEach(function (aCodeset) {
                codesetHash = typicalHash["" + aCodeset.id];
                codeTypes.forEach(function (aType) {
                    var typicalHashByType;

                    typicalHashByType = codesetHash[aType[config.CodeTypes_code_attributeName]];

                    if (triggerCodes) {
                        triggerTypeList = triggerCodes[aType[config.CodeTypes_code_attributeName]];
                        if (triggerTypeList && triggerTypeList.length) {
                            triggerTypeList.forEach(function (row) {
                                    var targetCodeEntry;
                                    if (row[config.episode_code_set_attributeName] === aCodeset.id) {

                                        if (typicalHashByType.hasOwnProperty(row[config.episode_code_value_attributeName])) {
                                            //i already know that code, so i just need to flip his trigger flag
                                            targetCodeEntry = typicalHashByType[row[config.episode_code_value_attributeName]];
                                            targetCodeEntry[config.episode_code_trigger_attributeName] = "Y";
                                        } else {
                                            //i can only count on the trigger field of this row so I need to clear the other fields
                                            row[config.episode_code_typical_attributeName] = "N";
                                            row[config.episode_code_complication_attributeName] = "N";
                                            row[config.episode_code_trigger_attributeName] = "Y";
                                            row[config.episode_code_exclusionType_code_attributeName] = "N";
                                            typicalHashByType[row[config.episode_code_value_attributeName]] = row;
                                        }
                                    }
                                }
                            );
                        }
                    }
                    if (complicationCodes) {
                        complicationTypeList = complicationCodes[aType[config.CodeTypes_code_attributeName]];
                        if (complicationTypeList && complicationTypeList.length) {
                            complicationTypeList.forEach(function (row) {
                                    var targetCodeEntry;
                                    if (row[config.episode_code_set_attributeName] === aCodeset.id) {

                                        if (typicalHashByType.hasOwnProperty(row[config.episode_code_value_attributeName])) {
                                            targetCodeEntry = typicalHashByType[row[config.episode_code_value_attributeName]];
                                            //i already know that code, so i just need to flip his complication
                                            // flag AND set his complication type
                                            targetCodeEntry[config.episode_code_complication_attributeName] = "Y";
                                            targetCodeEntry[config.episode_code_complicationType_code_attributeName]
                                                = row[config.episode_code_complicationType_code_attributeName];
                                        } else {
                                            //i can only count on the complication field of this row
                                            row[config.episode_code_typical_attributeName] = "N";
                                            row[config.episode_code_complication_attributeName] = "Y";
                                            row[config.episode_code_trigger_attributeName] = "N";
                                            row[config.episode_code_exclusionType_code_attributeName] = "N";
                                            typicalHashByType[row[config.episode_code_value_attributeName]] = row;
                                        }
                                    }
                                }
                            );
                        }
                    }
                    if (exclusionCodes) {
                        exclusionTypeList = exclusionCodes[aType[config.CodeTypes_code_attributeName]];
                        if (exclusionTypeList && exclusionTypeList.length) {
                            exclusionTypeList.forEach(function (row) {
                                    var targetCodeEntry;
                                    if (row[config.episode_code_set_attributeName] === aCodeset.id) {

                                        //now just because the code is in the exclusions data, it might not still be an exclusion...what
                                        // happens if they uncheck all the claim type fields?  I should block that from happening in
                                        // the gui, but if it happens I don't want to mark him as an exclusion as I'm persisting it....
                                        // so check for none set and log if you ever run into any such rows in the data.

                                        //ALSO check and see if all claim types are set....in which case we want to set the ALL
                                        // flag for martin too.

                                        if((row[config.episode_code_excludeOn_IP_flag_attributeName] === "Y")
                                            && (row[config.episode_code_excludeOn_OP_flag_attributeName] === "Y")
                                            && (row[config.episode_code_excludeOn_PB_flag_attributeName] === "Y")
                                            && (row[config.episode_code_excludeOn_RX_flag_attributeName] === "Y")) {
                                            row[config.episode_code_excludeOn_ALL_flag_attributeName] = "Y";
                                        } else {
                                            row[config.episode_code_excludeOn_ALL_flag_attributeName] = "N";
                                        }

                                        if((row[config.episode_code_excludeOn_IP_flag_attributeName] === "Y")
                                            || (row[config.episode_code_excludeOn_OP_flag_attributeName] === "Y")
                                            || (row[config.episode_code_excludeOn_PB_flag_attributeName] === "Y")
                                            || (row[config.episode_code_excludeOn_RX_flag_attributeName] === "Y")) {

                                            if (typicalHashByType.hasOwnProperty(row[config.episode_code_value_attributeName])) {
                                                targetCodeEntry = typicalHashByType[row[config.episode_code_value_attributeName]];
                                                //i already know that code, so i just need to flip his exclusion flag
                                                targetCodeEntry[config.episode_code_exclusionType_code_attributeName] = "C";
                                                targetCodeEntry[config.episode_code_excludeOn_ALL_flag_attributeName]
                                                    = row[config.episode_code_excludeOn_ALL_flag_attributeName];
                                                targetCodeEntry[config.episode_code_excludeOn_IP_flag_attributeName]
                                                    = row[config.episode_code_excludeOn_IP_flag_attributeName];
                                                targetCodeEntry[config.episode_code_excludeOn_OP_flag_attributeName]
                                                    = row[config.episode_code_excludeOn_OP_flag_attributeName];
                                                targetCodeEntry[config.episode_code_excludeOn_PB_flag_attributeName]
                                                    = row[config.episode_code_excludeOn_PB_flag_attributeName];
                                                targetCodeEntry[config.episode_code_excludeOn_RX_flag_attributeName]
                                                    = row[config.episode_code_excludeOn_RX_flag_attributeName];
                                            } else {
                                                //again, i can only count on the exclusion field of this rows
                                                row[config.episode_code_typical_attributeName] = "N";
                                                row[config.episode_code_complication_attributeName] = "N";
                                                row[config.episode_code_trigger_attributeName] = "N";
                                                //it has to be claim exclusion in 3.1
                                                row[config.episode_code_exclusionType_code_attributeName] = "C";
                                                //row already has the exclusionClaimTypes set so I don't need to set it here
                                                typicalHashByType[row[config.episode_code_value_attributeName]] = row;
                                            }
                                        } else {
                                            SASEpisodeManager.logger.error(
                                                "Discarding code in exclusions list that had no exclusion claim types specified... code:"
                                                + row[config.episode_code_value_attributeName]
                                                + " in codeset "
                                                + row[config.episode_code_set_attributeName]);
                                        }
                                    }
                                }
                            );
                        }
                    }
                });
            });

            //now collapse the tree of minihashes into one big array like the 3.1 model wants
            codeSets.forEach(function (aCodeset) {
                codesetHash = typicalHash["" + aCodeset.id];
                codeTypes.forEach(function (aType) {
                    var mergedTypeHash = codesetHash[aType[config.CodeTypes_code_attributeName]];

                    for (var code in mergedTypeHash) {
                        //noinspection JSUnfilteredForInLoop
                        newCodesList.push(mergedTypeHash[code]);
                    }
                });
            });
            internalForm.Codes = newCodesList;
            return internalForm;
        }

        //adrian wants the subtype info in tree form so the subtype id and description don't have to be repeated....
        //so convert from my table to that form here.
        function treeifySubtypeCodes(internalForm) {
            var hash = {},
                updateForm = [];

            internalForm.SubTypes.forEach(function (st) {
                var subTypeName = st.SUBTYPE_GROUP_NM,
                    thatSubType,
                    newSubType;

                if(subTypeName){
                    if(hash.hasOwnProperty(subTypeName)){
                        thatSubType = hash[subTypeName];
                        thatSubType.SubTypeCodes.push({
                            "CODE_VALUE": st.CODE_VALUE,
                            "CODE_SET_ID": st.CODE_SET_ID
                        });

                    } else {
                        newSubType = {"SUBTYPE_GROUP_NM": st.SUBTYPE_GROUP_NM,
                            "SUBTYPE_GROUP_DESC": st.SUBTYPE_GROUP_DESC,
                            "SubTypeCodes": [
                                {
                                    "CODE_VALUE": st.CODE_VALUE,
                                    "CODE_SET_ID": st.CODE_SET_ID
                                }
                            ]
                        };
                        hash[st.SUBTYPE_GROUP_NM] = newSubType;

                    }
                }
            });
            Object.keys(hash).forEach(function (sgname){
                updateForm.push(hash[sgname]);
            });
            internalForm.SubTypes = updateForm;
            return internalForm;
        }

        midTierFormOfModel = implodeCodes(midTierFormOfModel);
        midTierFormOfModel = treeifySubtypeCodes(midTierFormOfModel);
        return midTierFormOfModel;
    }

};
