// (c) 2015, SAS Institute Inc.
jQuery.sap.require('sas.hc.ui.layout.Masonry');
jQuery.sap.require('sas.hc.ui.layout.MasonryData');

jQuery.sap.declare("sas.ea.util.JobResults");


/**
 * Contains helper functions to handle Artifacts (results from running steps)
 */

sas.ea.util.JobResults = {

    // *********************************************************************
    // special UI array tracking START

    // since RunAll buttons are built where we don't have access to the data and
    // their IDs can't be data bound, we have to track them with their jobs

    _aRAs: [],      // array to track RunAll btn IDs to track RunAll jobs, since we can't data bind the ID nor can we change the ID

    // saspfj, TBDeleted
    //connectJobToRunAllBtn: function ( uidProject, uidJob, uidUi ) {
    //    //debugger;
    //
    //    // if job is already in list, then update the Ui ID
    //    for ( var x=0; x < sas.ea.util.JobResults._aRAs.length; ++x ) {
    //        //if ( sas.ea.util.JobResults._aRAs[x].uidUi === uidUi ) {
    //        if ( sas.ea.util.JobResults._aRAs[x].uidJob === uidJob ) {
    //            SEA.log( "UID", "Updated Job:" + uidJob + " to ui: " + uidUi );
    //            sas.ea.util.JobResults._aRAs[x].uidUi = uidUi;
    //            return;
    //        }
    //    }
    //
    //    // must be new, add it
    //    sas.ea.util.JobResults.addRunAllIDs( uidProject, uidJob, uidUi );
    //    SEA.log( "UID", "Connected RunAll Button:" + uidJob + " to ui: " + uidUi )
    //},

    // saspfj, RunAll, can probably remove these 3 methods
    addRunAllIDs: function ( uidProject, uidJob, uidUi ) {
        var oObj = { uidProject:uidProject, uidJob:uidJob, uidUi:uidUi };
        sas.ea.util.JobResults._aRAs.push( oObj );
        SEA.log( "UID", "addIDs: " + oObj.uidJob + " : " + oObj.uidUi );
    },

    getJobID4RaBtn: function ( uidUi ) {
        for ( var x=0; x < sas.ea.util.JobResults._aRAs.length; ++x ) {
            if ( sas.ea.util.JobResults._aRAs[x].uidUi === uidUi ) {
                return sas.ea.util.JobResults._aRAs[x].uidJob;
            }
        }
        return null;
    },

    getRaBtn4JobID: function ( uidJob ) {
        for ( var x=0; x < sas.ea.util.JobResults._aRAs.length; ++x ) {
            if ( sas.ea.util.JobResults._aRAs[x].uidJob === uidJob ) {
                return sas.ea.util.JobResults._aRAs[x].uidUi;
            }
        }
        return null;
    },


    // since jobs are asynchronous and UIs can come & go, we need to track UIs and Jobs
    // so when jobs finish we can check to see if the UI that started them is still
    // available, if so, we can populate it with the results

    _aIDs: [],      // array to track UI ids with project & job ids so we know what UI to update when jobs finish

    connectJobToUi: function ( uidProject, uidJob, aSteps, uidUi ) {
        // saspfj, determine why we are being called twice when opening a job
        //debugger;

        // if job is already in list, then update the Ui ID
        for ( var x=0; x < sas.ea.util.JobResults._aIDs.length; ++x ) {
            //if ( sas.ea.util.JobResults._aIDs[x].uidUi === uidUi ) {
            if ( sas.ea.util.JobResults._aIDs[x].uidJob === uidJob ) {
                SEA.log( "UID", "Updated Job:" + uidJob + " to ui: " + uidUi );
                sas.ea.util.JobResults._aIDs[x].uidUi = uidUi;
                return;
            }
        }

        // must be new, add it
        sas.ea.util.JobResults.addIDs( uidProject, uidJob, aSteps, uidUi );
        SEA.log( "UID", "Connected Job:" + uidJob + " to ui: " + uidUi );
    },

    addIDs: function ( uidProject, uidJob, aSteps, uidUi ) {
        var oObj = { uidProject:uidProject, uidJob:uidJob, aSteps:aSteps, uidUi:uidUi };
        sas.ea.util.JobResults._aIDs.push( oObj );
        SEA.log( "UID", "addIDs: " + oObj.uidJob + " : " + oObj.uidUi );
    },

    addStepIDforSharedEC: function ( uidJob, stepID, uidJobEC ) {
        //debugger;

        for ( var x=0; x < sas.ea.util.JobResults._aIDs.length; ++x ) {
            //debugger;
            if ( sas.ea.util.JobResults._aIDs[x].uidJob === uidJob ) {
                for ( var y=0; y < sas.ea.util.JobResults._aIDs[x].aSteps.length; ++ y ) {
                    if ( sas.ea.util.JobResults._aIDs[x].aSteps[y].progID === SC.STEP_KEY_EC ) {
                        sas.ea.util.JobResults._aIDs[x].aSteps[y].stepID = stepID;
                        // we'll also need to have the EC jobID for the proc print to run
                        sas.ea.util.JobResults._aIDs[x].uidJobEC = uidJobEC;
                    }
                }
            }
        }
    },

    // saspfj, think about refact to getProjectID
    getProjectUID: function ( uidUi ) {
        //debugger;
        for ( var x=0; x < sas.ea.util.JobResults._aIDs.length; ++x ) {
            if ( sas.ea.util.JobResults._aIDs[x].uidUi === uidUi ) {
                SEA.log( "UID", "FOUND - projectID:" + sas.ea.util.JobResults._aIDs[x].uidProject + " | " + uidUi );
                return sas.ea.util.JobResults._aIDs[x].uidProject;
            }
        }
        SEA.log( "UID", "NOT FOUND - projectID:? | " + uidUi );
        return null;
    },

    // saspfj, think about refact to getJobID
    getJobUID: function ( uidUi ) {
        for ( var x=0; x < sas.ea.util.JobResults._aIDs.length; ++x ) {
            if ( sas.ea.util.JobResults._aIDs[x].uidUi === uidUi ) {
                SEA.log( "UID", "FOUND - jobID:" + sas.ea.util.JobResults._aIDs[x].uidJob + " | " + uidUi );
                return sas.ea.util.JobResults._aIDs[x].uidJob;
            }
        }
        SEA.log( "UID", "NOT FOUND - jobID:? | " + uidUi );
        return null;
    },

    // saspfj, think about refact to getJobID
    getJobUIDEC: function ( uidUi ) {
        for ( var x=0; x < sas.ea.util.JobResults._aIDs.length; ++x ) {
            if ( sas.ea.util.JobResults._aIDs[x].uidUi === uidUi ) {
                SEA.log( "UID", "FOUND - jobID:" + sas.ea.util.JobResults._aIDs[x].uidJob + " | " + uidUi );
                // since "they" are asked for the EC jobID, try to give it to them, if we don't have it, return the regular jobID
                if ( typeof sas.ea.util.JobResults._aIDs[x].uidJobEC !== 'undefined' ) {
                    return sas.ea.util.JobResults._aIDs[x].uidJobEC;
                } else {
                    //return sas.ea.util.JobResults._aIDs[x].uidJob;
                    // S1219775, we're looking for the EC step jobID, we failed to find it above, use this as the fail-safe
                    // return value, since the uidJob above wasn't correct
                    return SEA.aRp[SEA.iCurrentTab].oJobs.jobs[0].jobID;
                }
            }
        }
        SEA.log( "UID", "NOT FOUND - jobID:? | " + uidUi );
        return null;
    },

    // saspfj, think about refact to getJobID
    getStepUID: function ( uidUi, stepKEY ) {
        for ( var x=0; x < sas.ea.util.JobResults._aIDs.length; ++x ) {
            if ( sas.ea.util.JobResults._aIDs[x].uidUi === uidUi ) {
                for ( var y=0; y < sas.ea.util.JobResults._aIDs[x].aSteps.length; ++ y ) {
                    if ( sas.ea.util.JobResults._aIDs[x].aSteps[y].progID === stepKEY ) {
                        SEA.log( "UID", "FOUND - stepID:" + sas.ea.util.JobResults._aIDs[x].aSteps[y].stepID + " | " + uidUi + "|" + stepKEY );
                        return sas.ea.util.JobResults._aIDs[x].aSteps[y].stepID;
                    }
                }
            }
        }

        // S1219775, we're looking for the EC step jobID, we failed to find it above, use this as the fail-safe
        // return value, since returning null causes problems for the EC step
        if ( stepKEY === SC.STEP_KEY_EC ) {
            if ( SEA.oActJob !== null ) {
                return SEA.oActJob.steps[0].stepID;
            }
        }

        SEA.log( "UID", "NOT FOUND - stepID:? | " + uidUi + "|" + stepKEY );
        return null;
    },

    // saspfj, think about refact to getUiID
    getUiUID: function ( uidJob ) {
        for ( var x = 0; x < sas.ea.util.JobResults._aIDs.length; ++x ) {
            if ( sas.ea.util.JobResults._aIDs[x].uidJob === uidJob ) {
                return sas.ea.util.JobResults._aIDs[x].uidUi;
            }
        }
        return null;
    },

    setVAparmsForUID: function ( uidUi, oVAparms ) {
        for ( var x=0; x < sas.ea.util.JobResults._aIDs.length; ++x ) {
            if ( sas.ea.util.JobResults._aIDs[x].uidUi === uidUi ) {
                sas.ea.util.JobResults._aIDs[x].oVAparms = oVAparms;
            }
        }
    },

    getVAparmsForUID: function ( uidUi ) {
        for ( var x=0; x < sas.ea.util.JobResults._aIDs.length; ++x ) {
            if ( sas.ea.util.JobResults._aIDs[x].uidUi === uidUi ) {
                //SEA.log( "UID", "FOUND - VAparms:" + sas.ea.util.JobResults._aIDs[x].uidJob + " | " + uidUi );
                return sas.ea.util.JobResults._aIDs[x].oVAparms;
            }
        }

        return null;
    },

    // special UI array tracking END
    // *********************************************************************


    populateStepUI: function ( uidJob, oStep ) {

        var uidUi = null;

        // find the UI UID for this job
        for ( var x = 0; x < sas.ea.util.JobResults._aIDs.length; ++x ) {
            if ( sas.ea.util.JobResults._aIDs[x].uidJob === uidJob ) {
                uidUi = sas.ea.util.JobResults._aIDs[x].uidUi;
                break;
            }
        }

        // if found, populate UI with artifacts
        if ( uidUi !== null ) {
            for ( var y = 0; y < oStep.lastExecution.artifacts.length; ++y ) {
                // saspfj, ToDo, determine why I commented this out
                //oStep.lastExecution.artifacts[y]
            }
        }
        // else nothing to do, no active UI to populate
    },

    // saspfj, ToDO, review this and the updateStepPanel() in Step.js to refactor & reuse common code
    renderStepUI: function ( projectID, stepInfo, bLocked, jobInfo, bRunAll ) {
       // debugger;
        var ctrl = null;
        SEA.log( "Render", "renderStepUI: " + stepInfo.progID + "-" + stepInfo.status + ", project:" + projectID + ", job:" + stepInfo.jobID );

        var stepKEY = stepInfo.progID;
        var uidUi = sas.ea.util.JobResults.getUiUID( stepInfo.jobID );

        // saspfj, temp simulation
        /*if ( stepInfo.progID === SC.STEP_KEY_VA ) {
            stepInfo.status = SC.STEP_STATUS_SUCCESS;
            stepInfo.lastExecution = {};
            stepInfo.lastExecution.status = SC.STEP_STATUS_SUCCESS;
            stepInfo.lastExecution.elapsed = 1234;
        }*/

        // lock handling, off by default
        if ( stepKEY === SC.STEP_KEY_EC ) {
            //sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-chkLock").setEnabled(false); //eslint-disable-line
            ctrl = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-chkLock");
            if ( ctrl ) {
                ctrl.setEnabled(false);
            }
        }

        var sTmrId = "#FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-timer";

        // saspfj, need to handle all states of status
        if ( (stepInfo.status === SC.STEP_STATUS_PENDING) && (stepKEY !== SC.STEP_KEY_VA) ) {
            //debugger;
            $(sTmrId).hide();

            // nothing to show if it's pending
            $("#FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-spnExpCol").hide();

            // if LV or PS is in a pending state, then be sure to collapse EC
            if ( (stepKEY === SC.STEP_KEY_LV) || (stepKEY === SC.STEP_KEY_PS) ) {
                sas.ea.util.Step.expcolShowPanel( false, uidUi, SC.STEP_KEY_EC );
            }

            // only procedd if the ctrl is there
            if (sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-image")) {
                if (stepKEY === SC.STEP_KEY_EC) {
                    sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-image").setSrc("html/resources/sas/ea/images/" + stepKEY + (bLocked ? "locked" : "") + ".png");

                    // S1218850, due to timing issues with redrawing active job, this can cause multiple EC steps to start
                    //           so for 3.1, removing auto-run feature
                    // added for Lina, if this is the first time through, then automatically run step
                    //if (stepInfo.executionCount === 0) {
                    //    setTimeout(function () {
                    //        sas.ea.util.Step.stepRunHandlerDOIT(uidUi, stepKEY);
                    //    }, 1000);   // SWAG
                    //}
                } else if (stepKEY === SC.STEP_KEY_PS) {
                    // enable the run button.  Leave the dialog closed in case they want to subset by index condition only.
                    sas.ea.util.JobResults.enableRunBtns(uidUi, stepKEY);

                } else {
                    sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-image").setSrc("html/resources/sas/ea/images/" + stepKEY + ".png");
                }
            }
            return;
        }

        switch ( stepInfo.status ) {
            // placeholder state, not currently used
            case SC.STEP_STATUS_DISABLED:
                return;

            case SC.STEP_STATUS_PENDING:
                // VA is special, even if it's pending, there may be artificats to render
                if ( stepInfo.progID === SC.STEP_KEY_VA ) {
                    sas.ea.util.JobResults.renderStepArtifacts_VA( stepInfo, uidUi, stepKEY, stepInfo.jobID );
                }
                return;

            case SC.STEP_STATUS_PROCESSING:
                // saspfj, TBD
                // there should already be a timer displayed, if not that means we are logging back in and a job is
                // still running, in this case we'll need to start/hook a timer into this control
                // FOR NOW, we'll show the timer cause if we're processing, then we need a timer...duh
                $(sTmrId).show();

                var iElapsed = 0; // saspfj, NOTE, elapsed time is in tenths of a second

                // we'll need to look up jobID to see if it's in an "active" job and get it's elapsed time
                for ( var x=0; x < SEA.aActiveJobs.length; ++ x ) {
                    if ( stepInfo.jobID === SEA.aActiveJobs[x].jobID) {
                        // if this job is still processing, make sure ALL the buttons are disabled
                        sas.ea.util.Step.disableRunButtons( uidUi, SC.STEP_KEY_LV );

                        // found it, if we haven't already started it, then start it
                        if ( !SEA.aActiveJobs[x].bStarted ) {
                            SEA.aActiveJobs[x].bStarted = true;
                            var dt = new Date();
                            iElapsed = (dt - SEA.aActiveJobs[x].tStart) / 100;
                            SEA.tmrStep.startStep( sTmrId, iElapsed );
                        }
                    }
                }
                //debugger;

                $("#FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-spnExpCol").hide();
                break;

            case SC.STEP_STATUS_SUCCESS:
            case SC.STEP_STATUS_WARNING:
            case SC.STEP_STATUS_ERROR:
                // add some smarts to our panels, we only want to have the "last" panel with new artifacts open
                // so we close the previous panel
                switch ( stepInfo.progID ) {
                    case SC.STEP_KEY_LV: sas.ea.util.Step.expcolShowPanel( false, uidUi, SC.STEP_KEY_EC ); break;
                    case SC.STEP_KEY_PS: sas.ea.util.Step.expcolShowPanel( false, uidUi, SC.STEP_KEY_EC ); break;
                    case SC.STEP_KEY_PA: sas.ea.util.Step.expcolShowPanel( false, uidUi, SC.STEP_KEY_LV ); break;
                    case SC.STEP_KEY_RA: sas.ea.util.Step.expcolShowPanel( false, uidUi, SC.STEP_KEY_PA ); break;
                    case SC.STEP_KEY_VA: sas.ea.util.Step.expcolShowPanel( false, uidUi, SC.STEP_KEY_RA ); break;
                }

                // now render the artifacts
                if ( stepInfo.progID === SC.STEP_KEY_VA ) {
                    // need to get the latest VAinfo
                    rs.ProjectService.getVAInfo( projectID ).resultHandler = function (result) {
                        if ( result !== "" ) {
                            //debugger;
                            SEA.aRp[SEA.iCurrentTab].vaInfo = $.parseJSON( result );
                            sas.ea.util.JobResults.renderStepArtifacts_VA( stepInfo, uidUi, stepKEY, stepInfo.jobID );
                        }
                    };

                    //sas.ea.util.JobResults.renderStepArtifacts_VA( stepInfo, uidUi, stepKEY, stepInfo.jobID );
                } else if ( stepInfo.progID === SC.STEP_KEY_PS ) {
                    //sas.ea.util.JobResults.renderStepArtifacts_PS( stepInfo, uidUi );
                    // for now, we'll try this guy, saspfj, ToDo, at some point we'll need to customize this for PS
                    sas.ea.util.JobResults.renderStepArtifacts_ECLVPARA( stepInfo, uidUi );
                } else {
                    sas.ea.util.JobResults.renderStepArtifacts_ECLVPARA( stepInfo, uidUi );
                }

                $(sTmrId).show();
                //ctrlPnlTmr.setText( "[" + SEA.tmrStep.getFriendlyTime(stepInfo.lastExecution.elapsed / 100) + "]" );
                //$(sTmrId).html( SEA.tmrStep.getFriendlyTime(stepInfo.lastExecution.elapsed * 10, true) );  // * 10, cause we're expecting tenths
                $(sTmrId).html( SEA.tmrStep.getFriendlyTime(stepInfo.lastExecution.elapsed, false) );
                //console.log("TODO-Update Panel:" + stepInfo.lastExecution.status + " in:" + (stepInfo.lastExecution.elapsed / 100) + " seconds");

                // update the icon, treat EC as special
                if ( (stepInfo.progID === SC.STEP_KEY_EC) && (stepInfo.status !== SC.STEP_STATUS_ERROR) ) {

                    // if this EC step is locked
                    if ( SEA.aRp[SEA.iCurrentTab].oRepoMeta.bLocked ) {
                        // remove run & lock buttons
                        sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepInfo.progID + "-btnRun").setVisible( false );
                        sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepInfo.progID + "-chkLock").setVisible( false );
                    } else {
                        // if we're not already locked, and we've run without errors, then enable the lock button
                        sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-chkLock").setEnabled(true);
                    }

                    // if this is a Construction job, then use colored backgrounds
                    //if ( SEA.aRp[SEA.iCurr].oJobs.jobs[x].jobType === "Construction" ) {
                    if ( sas.ea.util.JobResults.getJobObj(stepInfo.jobID).jobType === SC.JOB_TYPE_CONSTRUCTION ) {
                        sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepInfo.progID + "-image").setSrc("html/resources/sas/ea/images/" + stepInfo.progID + stepInfo.status + ".png");
                    } else {
                        // nope, this is an analysis job, use transparent
                        // saspfj, REFACTOR with udpateStepStatus() in step.js
                        if ( SEA.aRp[SEA.iCurrentTab].oRepoMeta.bLocked ) {
                            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepInfo.progID + "-image").setSrc("html/resources/sas/ea/images/EClocked.png");
                        } else {
                            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepInfo.progID + "-image").setSrc("html/resources/sas/ea/images/EC.png");
                        }

                    }
                } else {
                    sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepInfo.progID + "-image").setSrc("html/resources/sas/ea/images/" + stepInfo.progID + stepInfo.status + ".png");
                }
                break;
        }

        if ( stepKEY === SC.STEP_KEY_VA ) {
            // VA ignores the pending status
            if ( stepInfo.status !== SC.STEP_STATUS_PENDING ) {
                sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-status").setText(sas.ea.util.JobResults.getStepStatusTextVA(stepInfo.status));
            }
        } else if ( stepKEY === SC.STEP_KEY_EC ) {
            // saspfj, ToDo, make sure using current repo locked status is OK here
            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-status").setText(sas.ea.util.JobResults.getStepStatusText(stepInfo.status, SEA.aRp[SEA.iCurrentTab].oRepoMeta.bLocked));
        } else {
            // saspfj, ToDo, make sure using current repo locked status is OK here
            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-status").setText(sas.ea.util.JobResults.getStepStatusText(stepInfo.status, false));
        }

        // now reset the node
        // saspfj, ToDo, VERN, refactor with updateStepNode() in step.js - SINCE this code is sometimes
        // being called after flow is already rendered correctly, need to find out where this code is called
        // as the only path for rendering flow
        ctrl = sap.ui.getCore().byId("FLOW" + uidUi + "NODE" + stepKEY);
        if (ctrl) {
            // if jobInfo is null, then we're coming from a simple job event, which means we wouldn't be locking
            if ( jobInfo && ((jobInfo.jobType === SC.JOB_TYPE_ANALYSIS)||(jobInfo.jobType === SC.JOB_TYPE_SUBSET)) && (stepKEY === SC.STEP_KEY_EC) ) {
                if ( SEA.aRp[SEA.iCurrentTab].oRepoMeta.bLocked ) {
                    ctrl.setIcon("html/resources/sas/ea/images/" + stepKEY + "locked.png");
                } else {
                    ctrl.setIcon("html/resources/sas/ea/images/" + stepKEY + ".png");
                }
            } else {
                ctrl.setIcon("html/resources/sas/ea/images/" + stepKEY + stepInfo.status + "Node.png");
            }
        }
    },

    //if ( SEA.aRp[SEA.iCurr].oJobs.jobs[x].jobType === "Construction" ) {
    getJobObj: function ( jobID ) {
        for ( var x=0; x < SEA.aRp[SEA.iCurrentTab].oJobs.jobs.length; ++x ) {
            if ( SEA.aRp[SEA.iCurrentTab].oJobs.jobs[x].jobID === jobID ) {
                return SEA.aRp[SEA.iCurrentTab].oJobs.jobs[x];
            }
        }
        return null;
    },

    getStepStatusText: function ( status, bLocked ) {

        switch ( status ) {
            case SC.STEP_STATUS_DISABLED:   return oBundle.getText("STEP.STATUS.DISABLED");
            case SC.STEP_STATUS_PENDING:    return oBundle.getText("STEP.STATUS.PENDING");
            case SC.STEP_STATUS_PROCESSING: return oBundle.getText("STEP.STATUS.PROCESSING");
            case SC.STEP_STATUS_SUCCESS:    return oBundle.getText( bLocked ? "STEP.STATUS.LOCKED" : "STEP.STATUS.SUCCESS");
            case SC.STEP_STATUS_WARNING:    return oBundle.getText( bLocked ? "STEP.STATUS.LOCKED" : "STEP.STATUS.WARNING");
            case SC.STEP_STATUS_ERROR:      return oBundle.getText("STEP.STATUS.ERROR");
        }
    },

    getStepStatusTextVA: function ( status ) {

        switch ( status ) {
            case SC.STEP_STATUS_DISABLED:   return oBundle.getText("STEPVA.STATUS.DISABLED");
            case SC.STEP_STATUS_PENDING:    return oBundle.getText("STEPVA.STATUS.PENDING");
            case SC.STEP_STATUS_PROCESSING: return oBundle.getText("STEPVA.STATUS.PROCESSING");
            case SC.STEP_STATUS_SUCCESS:    return oBundle.getText("STEPVA.STATUS.SUCCESS");
            case SC.STEP_STATUS_WARNING:    return oBundle.getText("STEPVA.STATUS.WARNING");
            case SC.STEP_STATUS_ERROR:      return oBundle.getText("STEPVA.STATUS.ERROR");
        }
    },


    // START PDFalbum *********************************************************************
    //dlgPDFalbum: new sas.hc.m.Dialog({ id:"pdfvDlg", contentWidth:"98%", contentHeight:"96%", modal:true,
    dlgPDFalbum: new sas.hc.m.Dialog({ id:"pdfvDlg", modal:true, stretch:true,
        rightButton:new sas.hc.m.Button({
            text: oBundle.getText("Close.Button.txt"),
            press: function () {
                sas.ea.util.JobResults.dlgPDFalbum.close();
            }
        })
    }),

    jsviewPDFalbumDialog: null,

    // END PDFalbum *********************************************************************


    // START DATASETalbum *********************************************************************
    dlgDATASETalbum: new sas.hc.m.Dialog({ id:"datasetvDlg", modal:true, stretch:true,
        rightButton:new sas.hc.m.Button({
            text: oBundle.getText("Close.Button.txt"),
            press: function () {
                sas.ea.util.JobResults.dlgDATASETalbum.close();
            }
        })
    }),

    jsviewDATASETalbumDialog: null,

    // END DATASETalbum *********************************************************************


    renderStepArtifacts_ECLVPARA: function ( stepInfo, uidUi ) {
        //debugger;
        SEA.log( "Render", "renderStepArtifacts_ECLVPARA: " + stepInfo.progID );

        var stepKEY = stepInfo.progID;

        // process Reports, Data sets & all other artifacts
        var iR = 0;
        var iD = 0;
        var iO = 0;

        // clear out existing artifacts
        sas.ea.util.JobResults.clearStepArtifacts(uidUi, stepInfo.progID);

        // do a pass over the artifacts to get a count for the labels
        for (var x = 0; x < stepInfo.lastExecution.artifacts.length; ++x) {
            switch (stepInfo.lastExecution.artifacts[x].extension) {
                case "pdf":
                    ++iR;
                    break;
                case "sas7bdat":
                case "sas7bvew":
                    ++iD;
                    break;
                case "log":
                    // NOTE:  removing, since we're not showing the log in the production UI
                    ++iO; // saspfj, ToDo, remove before production
                    break;
                default:  // nothing to do, we don't care about unknown items
                    break;
            }
        }

        var ctrlGridR = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsR");
        var ctrlGridD = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsD");
        var ctrlGridO = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsO");

        // get Martin's JSON thingie & include any msgs from it
        // saspfj, ToDo, VERN, after Cummins guarantee's us a value JSON thingie
        if ( stepInfo.lastExecution.sasStatus === "" ) {
            //debugger;
            var oMstatus = {};
            oMstatus.messages = [];
            oMstatus.status = true;
        }else {
            var oMstatus = $.parseJSON(stepInfo.lastExecution.sasStatus);
        }
        iO += oMstatus.messages.length;

        // if we have artifacts, then allow user to expand/collapse
        if ((iR+iD+iO) > 0) {
            $("#FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-spnExpCol").show();
        }

        // don't add any section label when there's no artifacts for that section OR the job failed
        // NOTE:  for now, we'll pass 0 (zero) into the label gen method so numbers don't show up, if requested we can add this back easily
        if ( (iR > 0) && oMstatus.status ) {
            ctrlGridR.addContent(sas.ea.util.JobResults.genArtifactLabel(oBundle.getText("StepPanels.Artfct.Rpts.lbl"), 0, stepKEY, "R", uidUi, true, sas.ea.util.JobResults.viewAllPDFs));
            var ctrlGridR_M = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsR_M");
        }
        if ( (iD > 0) && oMstatus.status ) {
            // saspfj, change false to true after adding support to view all datasets
            ctrlGridD.addContent(sas.ea.util.JobResults.genArtifactLabel(oBundle.getText("StepPanels.Artfct.Data.lbl"), 0, stepKEY, "D", uidUi, true, sas.ea.util.JobResults.viewAllDATASETs));
            var ctrlGridD_M = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsD_M");
        }
        if (iO > 0) {
            ctrlGridO.addContent(sas.ea.util.JobResults.genArtifactLabel(oBundle.getText("StepPanels.Artfct.Exec.lbl"), 0, stepKEY, "O", uidUi, false, null));
            var ctrlGridO_M = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsO_M");
        }

        // add those msgs now
        for( var y=0; y < oMstatus.messages.length; ++y ) {
            ctrlGridO_M.addContent(sas.ea.util.JobResults.genStepOutputGridItemLog( uidUi, stepKEY, oMstatus.messages[y].text, null ));
        }

        // 3.2 LV, PA & RA all get the data mart execution note
        //ctrlGridO_M.addContent(sas.ea.util.JobResults.genStepOutputGridItemLog( uidUi, stepKEY, oBundle.getText("ExecutionNote.DataMart.txt"), null ));

        SEA.hPDFs[stepKEY] = [];
        SEA.hDATASETs[stepKEY] = [];
        for (var z = 0; z < stepInfo.lastExecution.artifacts.length; ++z) {
            //console.log("Artifact:" + stepInfo.lastExecution.artifacts[z].name + " type:" + stepInfo.lastExecution.artifacts[z].extension);
            switch (stepInfo.lastExecution.artifacts[z].extension) {
                case "pdf":
                    if ( oMstatus.status ) {
                        // only add if job succeeded
                        ctrlGridR_M.addContent(sas.ea.util.JobResults.genReportLink(uidUi, stepKEY, stepInfo.lastExecution.artifacts[z]));
                        SEA.hPDFs[stepKEY].push( stepInfo.lastExecution.artifacts[z] );
                    }
                    break;

                // unless we have SAS code handling to have proc print work on views, we do nothing with them.
                // NOTE:  handling does exist in genStepOutputGridItemLog() to render views
                case "sas7bvew":
                    break;

                case "sas7bdat":
                    if ( oMstatus.status ) {
                        // only add if job succeeded
                        ctrlGridD_M.addContent(sas.ea.util.JobResults.genDatasetLink(uidUi, stepKEY, stepInfo.lastExecution.artifacts[z]));
                        SEA.log( "artifacts", "z:" + z + ", " + stepInfo.lastExecution.artifacts[z].size );
                        SEA.hDATASETs[stepKEY].push( stepInfo.lastExecution.artifacts[z] );
                    }
                    break;
                case "log":
                    // S1186417, UI: remove sas.log from the results surfaced in the job Details and make needed changes to section
                    //ctrlGridO.addContent(sas.ea.util.JobResults.genStepOutputGridItemLog(uidUi, stepKEY, "sas.log", window.location.origin + SC.RESULTS_PATH + stepInfo.lastExecution.artifacts[z].path));     // saspfj, move magic string
                    break;
                default:  // nothing to do, we don't care about unknown items
                    break;
            }
        }

        // if we had any reports or datasets, then turn on export & ExpCol btn
        if ((iR > 0) || (iD > 0)) {
            //sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-btnExp").setEnabled( true );
            $("#FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-spnExpCol").show();
        } else {
            //sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-btnExp").setEnabled( false );
            $("#FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-spnExpCol").hide();
        }

        //// now we obviously have artifacts for this step, so make sure VA panel reflects it
        //var ctrlGridA = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_VAOutsA");
        //
        //// assume if we have the grid, then VA must be a step in this job, so continue
        //if ( ctrlGridA ) {
        //    switch (stepKEY) {
        //        case SC.STEP_KEY_EC:
        //            sas.ea.util.Step.dlgLoadLASRoptions.aRpts[0].bAvail = true;
        //            ctrlGridA.addContent( sas.ea.util.JobResults.genStepOutputGridItemVA( "", oBundle.getText("LoadLasrDialog.Episode") ) );
        //            break;
        //        case SC.STEP_KEY_LV:
        //            sas.ea.util.Step.dlgLoadLASRoptions.aRpts[1].bAvail = true;
        //            ctrlGridA.addContent( sas.ea.util.JobResults.genStepOutputGridItemVA( "", oBundle.getText("LoadLasrDialog.Services") ) );
        //            break;
        //        case SC.STEP_KEY_PA:
        //            sas.ea.util.Step.dlgLoadLASRoptions.aRpts[2].bAvail = true;
        //            sas.ea.util.Step.dlgLoadLASRoptions.aRpts[3].bAvail = true;
        //            sas.ea.util.Step.dlgLoadLASRoptions.aRpts[4].bAvail = true;
        //            ctrlGridA.addContent( sas.ea.util.JobResults.genStepOutputGridItemVA( "", oBundle.getText("LoadLasrDialog.Provider") ) );
        //            ctrlGridA.addContent( sas.ea.util.JobResults.genStepOutputGridItemVA( "", oBundle.getText("LoadLasrDialog.Patient") ) );
        //            ctrlGridA.addContent( sas.ea.util.JobResults.genStepOutputGridItemVA( "", oBundle.getText("LoadLasrDialog.Bundled") ) );
        //            break;
        //        case SC.STEP_KEY_RA: sas.ea.util.JobResults.genStepOutputGridItemVA( "", "" );
        //            sas.ea.util.Step.dlgLoadLASRoptions.aRpts[5].bAvail = true;
        //            sas.ea.util.Step.dlgLoadLASRoptions.aRpts[6].bAvail = true;
        //            ctrlGridA.addContent( sas.ea.util.JobResults.genStepOutputGridItemVA( "", oBundle.getText("LoadLasrDialog.BudCosts") ) );
        //            ctrlGridA.addContent( sas.ea.util.JobResults.genStepOutputGridItemVA( "", oBundle.getText("LoadLasrDialog.BdPyBdCs") ) );
        //            break;
        //    }
        //}
    },

    renderStepArtifacts_PS: function ( stepInfo, uidUi ) {
    },

    renderStepArtifacts_VA: function ( stepInfo, uidUi, stepKEY, jobID ) {
        SEA.log( "Render", "renderStepArtifacts_VA: " + stepInfo.progID );

        sas.ea.util.JobResults.clearStepArtifactsVA( uidUi, stepInfo.progID );

        // if we're PENDING, then nothing else to do
        if ( stepInfo.status === SC.STEP_STATUS_PENDING ) {
            return;
        }

        // "There can be only one"...job with VA output that is
        if ( jobID !== SEA.aRp[SEA.iCurrentTab].vaInfo.jobID ) {
            // obviously we aren't the job, so nothing else to do
            return;
        }

        // saspfj, ToDo, remove OutsA for VA, no longer needed with latest UX change
        var ctrlGridA = sap.ui.getCore().byId( "FLOW" + uidUi + "STEPPANEL_VAOutsA");
        var ctrlGridV = sap.ui.getCore().byId( "FLOW" + uidUi + "STEPPANEL_VAOutsV");
        var ctrlGridO = sap.ui.getCore().byId( "FLOW" + uidUi + "STEPPANEL_VAOutsO");

        // UX change, don't display available reports - they are available from the Create Reports dialog
        //// if we have any reports available to create, then render them
        //if ( sas.ea.util.Step.dlgLoadLASRoptions.areReportsAvailable() ) {
        //    ctrlGridA.addContent(sas.ea.util.JobResults.genArtifactLabel(oBundle.getText("VAStepPanel.Avail.lbl"), 0, stepKEY, "A", uidUi, false, null));
        //    var ctrlGridA_M = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsA_M");
        //    for ( var x=0; x < sas.ea.util.Step.dlgLoadLASRoptions.aRpts.length; ++x ) {
        //        if ( sas.ea.util.Step.dlgLoadLASRoptions.aRpts[x].bAvail ) {
        //            ctrlGridA_M.addContent(sas.ea.util.JobResults.genVAReportLabel(uidUi, sas.ea.util.Step.dlgLoadLASRoptions.aRpts[x].sName));
        //        }
        //    }
        //}

        // if we have warnings or errors, then display log
        if ( (stepInfo.status === SC.STEP_STATUS_WARNING) || (stepInfo.status === SC.STEP_STATUS_ERROR) ) {
            //debugger;

            ctrlGridO.addContent(sas.ea.util.JobResults.genArtifactLabel(oBundle.getText("StepPanels.Artfct.Exec.lbl"), 0, stepKEY, "O", uidUi, false, null));
            var ctrlGridO_M = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsO_M");

            var oMstatus = $.parseJSON(stepInfo.lastExecution.sasStatus);
            SEA.log( "VA", "oMstatus:" + stepInfo.lastExecution.sasStatus);
            for( var y=0; y < oMstatus.messages.length; ++y ) {
                ctrlGridO_M.addContent(sas.ea.util.JobResults.genStepOutputGridItemLog( uidUi, SC.STEP_KEY_VA, oMstatus.messages[y].text, null ));
            }
            // saspfj, ToDo, replace 0 with real value, using 0 to get LOG
            // saspfj, ToDo, VERN, GOOFLA, check path below
            //debugger;
            SEA.log( "VA", "artifacts count:" + stepInfo.lastExecution.artifacts.length );
            if ( stepInfo.lastExecution.artifacts.length > 0 ) {
                SEA.log( "VA", "artifacts 0:" + stepInfo.lastExecution.artifacts[0].path );
                // S1186417, UI: remove sas.log from the results surfaced in the job Details and make needed changes to section
                //ctrlGridO_M.addContent(sas.ea.util.JobResults.genStepOutputGridItemLog(uidUi, SC.STEP_KEY_VA, "sas.log", window.location.origin + SC.RESULTS_PATH + stepInfo.lastExecution.artifacts[0].path));     // saspfj, move magic string
            }
        }

        // if we have success or warnings, then render any reports
        if ( (stepInfo.status === SC.STEP_STATUS_SUCCESS) || (stepInfo.status === SC.STEP_STATUS_WARNING) ) {
            //debugger;

            if ( SEA.aRp[SEA.iCurrentTab].vaInfo.reports.length > 0 ) {
                // make call to get current VA output, if we're the "one", then populate the "Ready to View" items
                //ctrlGridV.addContent(sas.ea.util.JobResults.genArtifactLabel(oBundle.getText("VAStepPanel.View.lbl"), 0, stepKEY, "V", uidUi, false, null));
                //ctrlGridV.addContent(sas.ea.util.JobResults.genArtifactLabelMsg("These items are found at:", "Shared Data  >  SAS Episode Analytics 3.1  >  Reports  >  Repository 1", stepKEY, "V", uidUi));

                var sFoundAt = sas.ea.util.JobResults.genVAreportsLocation();
                ctrlGridV.addContent(sas.ea.util.JobResults.genArtifactLabelMsg( oBundle.getText("VA_ItemsFound.txt"), sFoundAt, stepKEY, "V", uidUi));
                var ctrlGridV_M = sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsV_M");

                var vaInfo = SEA.aRp[SEA.iCurrentTab].vaInfo;
                for (var z = 0; z < vaInfo.reports.length; ++z) {
                    // the report name is the last part of the path
                    var sName = vaInfo.reports[z];
                    sName = sName.substr(sName.lastIndexOf("/")+1);
                    //ctrlGridV.addContent(sas.ea.util.JobResults.genVAReportGridItemVA(sName, vaInfo.reportKey, vaInfo.reports[z]));
                    SEA.log( "VA", "Report " + z + ": " + sName + ", " + vaInfo.reportKey + ", " + vaInfo.reports[z] );

                    // for 3.1 we're not doing any data explorations, so ignore it
                    if ( sName !== "SEA Data Exploration") {
                        ctrlGridV_M.addContent(sas.ea.util.JobResults.genVAReportGridItemVA(sName, vaInfo.reportKey, vaInfo.reports[z]));
                    }

                    //debugger;
                    //var sPath = vaInfo.reports[z];
                    //sPath = sPath.substr(0, sPath.lastIndexOf("/"));
                    //ctrlGridV.addContent(sas.ea.util.JobResults.genVAReportGridItemVA(sName, vaInfo.reportKey, sPath));
                }
            }
        }
    },

    genVAreportsLocation: function () {
        var sTxt = "";

        var sPath = SEA.aRp[SEA.iCurrentTab].vaInfo.reports[0];
        if ( sPath.length > 0 ) {
            // first let's strip off the leading /
            sTxt = sPath.substr( 1 );

            // then strip off the last / and report name
            sTxt = sTxt.substr( 0, sTxt.lastIndexOf('/') );

            // now substitute the other / for sp>sp
            sTxt = sTxt.replace(/\//g, " > ");
        } else {
            // fallback, which "should" never happen
            sTxt += oBundle.getText("VA_ReportLocation.txt") + (parseInt(SEA.iCurrentTab) + 1);
        }

        return sTxt;
    },

    genArtifactLabel: function ( sLabel, iCnt, stepKEY, sGrid, uidUi, bViewAll, funcViewAll ) {

        // empty labels means no bold label and no divider, a simpler container used by VA
        if ( sLabel === "" ) {
            return new sap.ui.layout.VerticalLayout({
                layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"}),
                width: "100%",
                content: [
                    new sas.hc.ui.layout.Masonry({
                        id: "FLOW" + uidUi + "STEPPANEL_" + stepKEY + "Outs" + sGrid + "_M",
                        options: {"gutter": 32},
                        content: []
                    })
                ]
            });
        } else {
            return new sap.ui.layout.VerticalLayout({
                layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"}),
                width: "100%",
                content: [
                    //new sap.ui.commons.HorizontalDivider({ height:sap.ui.commons.HorizontalDividerHeight.Ruleheight })
                    new sap.ui.layout.HorizontalLayout({
                        width: "100%",
                        content: [
                            new sas.ea.util.JobResults.genBoldLabel(sLabel + ((iCnt > 0) ? " - " + iCnt : "")),
                            new sas.hc.m.Link({
                                text: bViewAll ? oBundle.getText("viewall.link.txt") : "",
                                press: function () {
                                    funcViewAll(uidUi, stepKEY);
                                }
                            })
                        ]
                    }),
                    new sap.ui.commons.HorizontalDivider({height: sap.ui.commons.HorizontalDividerHeight.Ruleheight}),
                    new sas.hc.ui.layout.Masonry({
                        id: "FLOW" + uidUi + "STEPPANEL_" + stepKEY + "Outs" + sGrid + "_M",
                        options: {"gutter": 32},
                        content: []
                    })
                ]
            });
        }
    },

    genArtifactLabelMsg: function ( sLabel1, sLabel2, stepKEY, sGrid, uidUi ) {

        return new sap.ui.layout.VerticalLayout({
            layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"}),
            width: "100%",
            content: [
                //new sap.ui.commons.HorizontalDivider({ height:sap.ui.commons.HorizontalDividerHeight.Ruleheight })
                new sap.ui.layout.HorizontalLayout({
                    width: "100%",
                    content: [
                        new sas.ea.util.JobResults.genLabel( sLabel1 ),
                        new sas.ea.util.JobResults.genBoldLabel( sLabel2 )
                    ]
                }),
                new sas.hc.ui.layout.Masonry({
                    id: "FLOW" + uidUi + "STEPPANEL_" + stepKEY + "Outs" + sGrid + "_M",
                    options: {"gutter": 32},
                    content: []
                })
            ]
        });
    },

    genBoldLabel: function ( sLabel ) {
        var lbl = new sas.hc.m.Label( {text:sLabel, wrapping:false} );
        lbl.setDesign( sap.ui.commons.LabelDesign.Bold );
        return lbl;
    },

    genLabel: function ( sLabel ) {
        //noinspection UnnecessaryLocalVariableJS
        var lbl = new sas.hc.m.Label( {text:sLabel, wrapping:false} );
        return lbl;
    },

    viewAllPDFs: function ( uidUi, stepKEY ) {
        // debugger;

        sas.ea.util.JobResults.dlgPDFalbum.setTitle("dummy");
        sas.ea.util.JobResults.dlgPDFalbum.aPDFs = SEA.hPDFs[stepKEY];

        // out with the old
        sas.ea.util.JobResults.dlgPDFalbum.destroyContent();

        // shouldn't have to, but we do anyway
        if (sas.ea.util.JobResults.jsviewPDFalbumDialog !== null) {
            sas.ea.util.JobResults.jsviewPDFalbumDialog.destroy();
        }

        // in with the new
        sas.ea.util.JobResults.jsviewPDFalbumDialog = new sap.ui.jsview("sas.ea.view.PDFalbum");
        sas.ea.util.JobResults.dlgPDFalbum.addContent(sas.ea.util.JobResults.jsviewPDFalbumDialog);

        // open the dialog and load the PDFs
        sas.ea.util.JobResults.dlgPDFalbum.open();
        sas.ea.util.JobResults.dlgPDFalbum.loadPDFs( stepKEY );
    },

    viewAllDATASETs: function ( uidUi, stepKEY ) {
        //debugger;

        sas.ea.util.JobResults.dlgDATASETalbum.setTitle("dummy");
        sas.ea.util.JobResults.dlgDATASETalbum.aDATASETs = SEA.hDATASETs[stepKEY];

        var aDATASETs = sas.ea.util.JobResults.dlgDATASETalbum.aDATASETs;   // simplify code below

        // we have the names & after the jobs have run, we'll have the URLs
        for ( var x=0; x < aDATASETs.length; ++x ) {
            // for now though, we set the URLs to ""
            aDATASETs[x].sUrl = "";
        }


        // out with the old
        sas.ea.util.JobResults.dlgDATASETalbum.destroyContent();

        // shouldn't have to, but we do anyway
        if (sas.ea.util.JobResults.jsviewDATASETalbumDialog !== null) {
            sas.ea.util.JobResults.jsviewDATASETalbumDialog.destroy();
        }

        // in with the new
        sas.ea.util.JobResults.jsviewDATASETalbumDialog = new sap.ui.jsview("sas.ea.view.DATASETalbum");

        sas.ea.util.JobResults.dlgDATASETalbum.addContent(sas.ea.util.JobResults.jsviewDATASETalbumDialog);


        // before we open it, let's make sure it has all the info it needs
        sas.ea.util.JobResults.dlgDATASETalbum.projectID = sas.ea.util.JobResults.getProjectUID(uidUi);

        // if we're an EC step, get the shared EC jobID, else get the regular jobID
        if ( stepKEY === SC.STEP_KEY_EC ) {
            sas.ea.util.JobResults.dlgDATASETalbum.jobID = sas.ea.util.JobResults.getJobUIDEC(uidUi);
        } else {
            sas.ea.util.JobResults.dlgDATASETalbum.jobID = sas.ea.util.JobResults.getJobUID(uidUi);
        }

        sas.ea.util.JobResults.dlgDATASETalbum.stepID = sas.ea.util.JobResults.getStepUID( uidUi, stepKEY );


        // we know we're going to start off with 0
        sas.ea.util.JobResults.dlgDATASETalbum.iInd = 0;
        sas.ea.util.JobResults.dlgDATASETalbum.runProcPrint( 0, aDATASETs[0].name, SC.PRIORITY_HIGH );

        // now let's finish the rest of them, giving the one above and the code below a slight head-start
        setTimeout( function() {
            for ( var y=1; y < aDATASETs.length; ++y ) {
                sas.ea.util.JobResults.dlgDATASETalbum.runProcPrint( y, aDATASETs[y].name, SC.PRIORITY_NORMAL );
            }
        }, 700 );   // empirical #

        // open the dialog and load the DATASETs
        sas.ea.util.JobResults.dlgDATASETalbum.open();
        sas.ea.util.JobResults.dlgDATASETalbum.loadDATASETs( stepKEY );
    },

    updateStepUI: function ( projectID, jobID, stepID, stepKEY, invoice, bRunAll ) {
        //debugger;
        SEA.log( "Render", "updateStepUI: " + projectID + ":" + jobID, + ":" + stepID, + ":" + stepKEY );

        rs.ProjectService.getStep( projectID, jobID, stepID ).resultHandler = function (sResult) {
            //debugger;
            if (sResult !== "") {
                //debugger;
                var stepInfo = $.parseJSON(sResult);
                SEA.log( "stepInfo", sResult );
                sas.ea.util.JobResults.renderStepUI( projectID, stepInfo, SEA.aRp[SEA.iCurrentTab].oRepoMeta.bLocked, null, bRunAll );
            }
            // saspfj, turn run button(s) back on ???
        };
    },

    genVAReportGridItemVA: function ( sReport, sKey, sPath ) {

        return new sap.ui.layout.HorizontalLayout({
            content: [
                new sap.ui.layout.VerticalLayout({
                    content: [
                        new sas.hc.m.Image({
                            src: "html/resources/sas/ea/images/SASREPORT2Gimage.png", width:"26px", height:"30px",
                            press: function (e) {
                                // kludge for 3.1, we have 1 data exploration and it needs to be opened with VAE
                                if ( sReport === "SEA Data Exploration") {
                                    SEA.log("App", "I appSwitcher.openObject - report:" + sReport + ", key:" + sKey + ", path:" + sPath + ", app:" + "VisualAnalyticsExplorerLogon");
                                    sas.applicationSwitcher.openObject("Report.BI", {
                                        targetReposKey: sKey,
                                        targetNavPath: sPath
                                    }, "VisualAnalyticsExplorerLogon");
                                    //}, "VisualAnalyticsDesignerLogon");
                                } else {
                                    SEA.log("App", "I appSwitcher.openObject - report:" + sReport + ", key:" + sKey + ", path:" + sPath);
                                    sas.applicationSwitcher.openObject("Report.BI", {
                                        targetReposKey: sKey,
                                        targetNavPath: sPath
                                        //}, "VisualAnalyticsExplorerLogon");
                                    });
                                }
                            }
                            //press: function (e) {
                            //    if ( e.shiftKey ) {
                            //        var sUrl = window.location.origin;
                            //        sUrl += "/SASVisualAnalyticsViewer/VisualAnalyticsViewer.jsp";
                            //        sUrl += "?reportName=" + sReport.replace(/ /gi, "+");
                            //        sPath = sPath.substr(0, sPath.lastIndexOf("/"));
                            //        sUrl += "&reportPath=" + sPath.replace(/ /gi, "+");
                            //        window.open(sUrl, "");
                            //    } else {
                            //        SEA.log("VA", "appSwitcher.openObject - report:" + sReport + ", key:" + sKey + ", path:" + sPath);
                            //        sas.applicationSwitcher.openObject("Report.BI", {
                            //            targetReposKey: sKey,
                            //            targetNavPath: sPath
                            //        });
                            //    }
                            //}
                        }),
                        new sas.hc.m.Image( {src: "html/resources/sas/ea/images/verticalPadding12.png", width:"12px", height:"12px"} )
                    ]
                }),
                new sap.ui.layout.VerticalLayout({
                    content: [
                        new sas.hc.m.Link({
                            text:sReport,
                            press: function (e) {
                                //SEA.log( "App", "L appSwitcher.openObject - report:" + sReport + ", key:" + sKey + ", path:" + sPath );
                                //sas.applicationSwitcher.openObject( "Report.BI", {
                                //    targetReposKey: sKey,
                                //    targetNavPath: sPath
                                //}, "VisualAnalyticsExplorerLogon");

                                // kludge for 3.1, we have 1 data exploration and it needs to be opened with VAE
                                if ( sReport === "SEA Data Exploration") {
                                    SEA.log("App", "L appSwitcher.openObject - report:" + sReport + ", key:" + sKey + ", path:" + sPath + ", app:" + "VisualAnalyticsExplorerLogon");
                                    sas.applicationSwitcher.openObject("Report.BI", {
                                        targetReposKey: sKey,
                                        targetNavPath: sPath
                                    }, "VisualAnalyticsExplorerLogon");
                                    //}, "VisualAnalyticsDesignerLogon");
                                } else {
                                    SEA.log("App", "L appSwitcher.openObject - report:" + sReport + ", key:" + sKey + ", path:" + sPath);
                                    sas.applicationSwitcher.openObject("Report.BI", {
                                        targetReposKey: sKey,
                                        targetNavPath: sPath
                                        //}, "VisualAnalyticsExplorerLogon");
                                    });
                                }
                            }

                            //press:function () {
                            //    var sUrl = window.location.origin;
                            //    sUrl += "/SASVisualAnalyticsViewer/VisualAnalyticsViewer.jsp";
                            //    sUrl += "?reportName=" + sReport.replace(/ /gi, "+");
                            //    sPath = sPath.substr(0, sPath.lastIndexOf("/"));
                            //    sUrl += "&reportPath=" + sPath.replace(/ /gi, "+");
                            //    window.open(sUrl, "");
                            //}

                            //press: function (e) {
                            //    if ( e.shiftKey ) {
                            //        var sUrl = window.location.origin;
                            //        sUrl += "/SASVisualAnalyticsViewer/VisualAnalyticsViewer.jsp";
                            //        sUrl += "?reportName=" + sReport.replace(/ /gi, "+");
                            //        sPath = sPath.substr(0, sPath.lastIndexOf("/"));
                            //        sUrl += "&reportPath=" + sPath.replace(/ /gi, "+");
                            //        window.open(sUrl, "");
                            //    } else {
                            //        SEA.log("VA", "appSwitcher.openObject - report:" + sReport + ", key:" + sKey + ", path:" + sPath);
                            //        sas.applicationSwitcher.openObject("Report.BI", {
                            //            targetReposKey: sKey,
                            //            targetNavPath: sPath
                            //        });
                            //        //}, "VisualAnalyticsDesignerLogon");       // if you want to specify the VA Designer to open instead
                            //    }
                            //}
                        }),
                        new sas.hc.m.Image( {src: "html/resources/sas/ea/images/verticalPadding12.png", width:"12px", height:"12px"} )
                    ]
                })
            ]
        });
    },

    genPrettySize: function ( lSize ) {

        /* OLD code
        var lK = 1024;
        var lM = 1024 * 1024;
        var lG = 1024 * 1024 * 1024;
        var lT = 1024 * 1024 * 1024 * 1024;
        var sSuff = "";

        if ( lSize < lK ) {
            lSize = lSize;
            sSuff = " bytes";
        } else if ( (lSize >= lK) && (lSize < lM) ) {
            lSize = lSize / lK;
            sSuff = " KB";
        } else if ( (lSize >= lM) && (lSize < lG) ) {
            lSize = lSize / lM;
            sSuff = " MB";
        } else if ( (lSize >= lG) && (lSize < lT) ) {
            lSize = lSize / lG;
            sSuff = " GB";
        } else {
            lSize = lSize / lT;
            sSuff = " TB";
        }

        return Math.round(lSize).toString() + sSuff;
        */

        // work-around for -1 size for > 2GB files
        if ( lSize === -1 ) {
            return "> 2 GB";
        }

        var aSufx = [ "B", "KB", "MB", "GB", "TB", "PB" ];
        // JS doesn't have a log() where you can specify base, so doing this based on: http://stackoverflow.com/questions/3019278/how-can-i-specify-the-base-for-math-log-in-javascript
        // changed round to trunc, since round didn't give the correct answers
        //var iLog = Math.trunc( Math.log(lSize) / Math.log(1024) );
        // change trunc to floor, since IE11 doesn't support trunc
        var iLog = Math.floor( Math.log(lSize) / Math.log(1024) );
        var iValue;

        if (lSize > 0) {
            //iValue = (int)((double)lSize / Math.Pow(1024, (double)iLog));
            iValue = Math.round( lSize / Math.pow(1024, iLog) );
        } else {
            iValue = Math.round( lSize );
            iLog = 0;
        }

        SEA.log( "misc", "convert " + lSize + " to " + iValue + aSufx[iLog]);
        return iValue + " " + aSufx[iLog];
    },

    genReportLink: function ( uidUi, stepKEY, oOut ) {
        var sType = oOut.extension;
        var sData = oOut.name;
        var sLabel = oOut.name.replace(/_/gi, " "); // change underscores to spaces for better label wrapping
        var sUrl = window.location.origin + SC.RESULTS_PATH + oOut.path;

        return new sap.ui.layout.HorizontalLayout({
            content: [
                new sap.ui.layout.VerticalLayout({
                    content: [
                        new sas.hc.m.Image({ src:"html/resources/sas/ea/images/PDFFILEimage.png", width:"26px", height:"30px", press:function () {
	                          console.log("Add MT call here - 5.1, report icon: " + SEA.aRp[SEA.iCurrentTab].oRepoMeta.id + ", " + sData);
	                          rs.auditDataView( SEA.aRp[SEA.iCurrentTab].oRepoMeta.id, sData+".pdf", null );
	                          window.open(sUrl, "");
                        } }),
                        new sas.hc.m.Image({ src:"html/resources/sas/ea/images/verticalPadding12.png", width:"12px", height:"12px"} )
                    ]
                }),
                new sap.ui.layout.VerticalLayout({
                    content: [
                        new sas.hc.m.Link({
                            text: sLabel,
                            press: function () {
	                              console.log("Add MT call here - 5.2, report icon: " + SEA.aRp[SEA.iCurrentTab].oRepoMeta.id + ", " + sData);
	                              rs.auditDataView( SEA.aRp[SEA.iCurrentTab].oRepoMeta.id, sData+".pdf", null );
                                window.open(sUrl, "");
                            }
                        }),
                        new sas.hc.m.Image( {src: "html/resources/sas/ea/images/verticalPadding12.png", width:"12px", height:"12px"} )
                    ]
                })
            ]
        });
    },

    genDatasetLink: function ( uidUi, stepKEY, oOut ) {
        var sType = oOut.extension;
        var sData = oOut.name;
        var sLabel = oOut.name.replace(/_/gi, " "); // change underscores to spaces for better label wrapping
        var sUrl = window.location.origin + SC.RESULTS_PATH + oOut.path;

        return new sap.ui.layout.VerticalLayout({
            //layoutData: new sap.ui.layout.GridData({span: "L2 M3 S6"}),
            content: [
                new sap.ui.layout.HorizontalLayout({
                    content: [
                        new sas.hc.m.Image({
                            src:    "html/resources/sas/ea/images/SASDATASETimage.png", width:"26px", height:"30px",
                            press:  function () {
                                sas.ea.util.Step.showDataPreviewDialog( uidUi, stepKEY, sData, sLabel, sUrl );
                            }
                        }),
                        new sap.ui.core.HTML({
                            content:    "<span style='font-size:small; font-variant:small-caps;'>" + sas.ea.util.JobResults.genPrettySize(oOut.size) + ((sType === "sas7bvew")? " view" : "") + "</span>",  // saspfj, temp string until we get a real view icon
                            tooltip:    oBundle.getText("Results.ClickToOpen"),
                            press:      function () {
                                sas.ea.util.Step.showDataPreviewDialog( uidUi, stepKEY, sData, sLabel, sUrl );
                            }
                        })
                    ]
                }),
                new sap.ui.layout.VerticalLayout({
                    content: [
                        new sas.hc.m.Link({
                            text:   sLabel,
                            press:  function () {
                                sas.ea.util.Step.showDataPreviewDialog(uidUi, stepKEY, sData, sLabel, sUrl);
                            }
                        }),
                        new sas.hc.m.Image( {src: "html/resources/sas/ea/images/verticalPadding18.png", width:"12px", height:"18px"} )
                    ]
                })
            ]
        });

    },

    genVAReportLabel: function ( uidUi, sLabel ) {

        return new sap.ui.layout.VerticalLayout({
            content: [
                new sas.hc.m.Image( {src: "html/resources/sas/ea/images/verticalPadding6.png", width:"12px", height:"6px"} ),
                new sas.hc.m.Label({ text:sLabel })
            ]
        });
    },

    genStepOutputGridItemLog: function ( uidUi, stepKEY, sMsg, sLink ) {

        // saspfj, refactor this
        if ( sLink != null ) {
            return new sap.ui.layout.VerticalLayout({
                layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"}),
                content: [
                    new sap.ui.core.HTML({
                        content:"<a href='" + sLink + "' target='_blank'>" + sMsg + "</a>"
                    })
                ]
            });
        } else {
            return new sap.ui.layout.VerticalLayout({
                layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"}),
                content: [
                    new sap.ui.core.HTML({
                        content:"<span>" + sMsg + "</span>"
                    })
                ]
            });
        }
    },

    enableRunBtns: function ( uidUi, stepKEY ) {
        //debugger;
        SEA.log( "Render", "enableRunBtns: " + stepKEY );

        sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-btnRun").setEnabled( true );

        // PS doesn't use a RunAll
        if ( stepKEY === SC.STEP_KEY_PS ) {
            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-btnRunAll").setEnabled( false );
        }
    },

    // SEE COMMENT for next method
    //// returns previous step of the passed step
    //// NOTE:  does not work for the VA step, will return null
    ////        EC also returns null since there isn't a previous step
    //getPreviousStep: function ( stepKEY ) {
    //
    //    switch( stepKEY ) {
    //        case SC.STEP_KEY_EC:    return null;
    //        case SC.STEP_KEY_LV:    return SC.STEP_KEY_EC;
    //        case SC.STEP_KEY_PS:    return SC.STEP_KEY_EC;
    //        case SC.STEP_KEY_PA:    return SC.STEP_KEY_LV;
    //        case SC.STEP_KEY_RA:    return SC.STEP_KEY_PA;
    //    }
    //
    //    // should never get here...
    //    return null;
    //},

    enableRunBtnsForParmSave: function ( uidUi, stepKEY ) {
        // for 3.1 since we're allowing users to re-run successfully run steps, there's no need to enable the Run
        // button after parms have changed.  The Run button enablement will already be properly set, so nothing
        // to do here, leaving code in but commented so in next release when we stop this nonsense of re-running
        // successfully run steps, we'll need this code
        // ALSO: above method also commented out, but left in for future use
        return;

        ////debugger;
        //SEA.log( "Render", "enableRunBtnsForParmSave: " + stepKEY );
        //
        //// enable the Run button for this step, only if previous step has been run sucessfully
        //var sStep = sas.ea.util.JobResults.getPreviousStep( stepKEY );
        //// if there is a previous step
        //if ( sStep ) {
        //    var sCurrStepID = "FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-status";
        //    var sPrevStepID = "FLOW" + uidUi + "STEPPANEL_" + sStep + "-status";
        //    // if previous step was success, then enable the run button for the step whose parms were changed
        //    if ( sap.ui.getCore().byId(sPrevStepID).getText() === oBundle.getText("STEP.STATUS.SUCCESS") ) {
        //        sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-btnRun").setEnabled(true);
        //    }
        //}
        //
        //// always enable Run All
        //// NOTE:  PS doesn't use a RunAll
        //if ( stepKEY !== SC.STEP_KEY_PS ) {
        //    sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-btnRunAll").setEnabled( false );
        //}
    },

    resetStepsUI: function ( uidUi, stepKEY ) {
        //debugger;
        SEA.log( "Render", "resetStepsUI: " + stepKEY );

        // here we want to do all downstream steps, so no breaks
        //noinspection FallThroughInSwitchStatementJS
        switch ( stepKEY ) {
            case SC.STEP_KEY_EC:    sas.ea.util.JobResults.clearStepArtifacts( uidUi, SC.STEP_KEY_EC );     sas.ea.util.JobResults.resetStepStatus( uidUi, SC.STEP_KEY_EC );
            case SC.STEP_KEY_LV:    sas.ea.util.JobResults.clearStepArtifacts( uidUi, SC.STEP_KEY_LV );     sas.ea.util.JobResults.resetStepStatus( uidUi, SC.STEP_KEY_LV );
            case SC.STEP_KEY_PS:    sas.ea.util.JobResults.clearStepArtifacts( uidUi, SC.STEP_KEY_PS );     sas.ea.util.JobResults.resetStepStatus( uidUi, SC.STEP_KEY_PS );
            case SC.STEP_KEY_PA:    sas.ea.util.JobResults.clearStepArtifacts( uidUi, SC.STEP_KEY_PA );     sas.ea.util.JobResults.resetStepStatus( uidUi, SC.STEP_KEY_PA );
            case SC.STEP_KEY_RA:    sas.ea.util.JobResults.clearStepArtifacts( uidUi, SC.STEP_KEY_RA );     sas.ea.util.JobResults.resetStepStatus( uidUi, SC.STEP_KEY_RA );
            case SC.STEP_KEY_VA:    sas.ea.util.JobResults.clearStepArtifacts( uidUi, SC.STEP_KEY_VA );
        }

        // and we need to disable all downstream run buttons
        //noinspection FallThroughInSwitchStatementJS
        switch ( stepKEY ) {
            case SC.STEP_KEY_EC:    sap.ui.getCore().byId( "FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_LV + "-btnRun").setEnabled( false ); sap.ui.getCore().byId( "FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_PS + "-btnRun").setEnabled( false );
            case SC.STEP_KEY_LV:    sap.ui.getCore().byId( "FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_PA + "-btnRun").setEnabled( false );
            case SC.STEP_KEY_PA:    sap.ui.getCore().byId( "FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_RA + "-btnRun").setEnabled( false );
            case SC.STEP_KEY_RA:    sap.ui.getCore().byId( "FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_VA + "-btnRun").setEnabled( sas.ea.util.Step.dlgLoadLASRoptions.areReportsAvailable(uidUi) );
        }

        // except for VA which we only want to clear artifacts for the passed step & downstream
        sas.ea.util.JobResults.clearStepArtifactsVA( uidUi, stepKEY );
        //sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_VA + "-btnRun").setEnabled(sas.ea.util.Step.dlgLoadLASRoptions.areReportsAvailable());
    },

    resetStepStatus: function ( uidUi, stepKEY ) {

        var aSteps = sas.ea.util.JobResults.getJobObj( sas.ea.util.JobResults.getJobUID(uidUi) ).steps;
        for ( var y=0; y < aSteps.length; ++y ) {
            if ( aSteps[y].progID === stepKEY ) {
                aSteps[y].status = SC.STEP_STATUS_PENDING;
                break;
            }
        }
    },

    clearStepArtifacts: function ( uidUi, stepKEY ) {
        //debugger;
        SEA.log("Render", "Clearing artifacts for: " + stepKEY);

        // if no artifacts, no need to expand/collapse
        $("#FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-spnExpCol").hide();

        if (stepKEY === SC.STEP_KEY_VA) {
            sas.ea.util.JobResults.clearStepArtifactsVA(uidUi, stepKEY);
        } else {
            sas.ea.util.JobResults.clearStepArtifactsECLVPARA(uidUi, stepKEY);
        }

        //// if artifacts are cleared, then nothing to export, VA does not have export
        //if (stepKEY !== SC.STEP_KEY_VA) {
        //    sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-btnExp").setEnabled(false);
        //}

        // and the status should be reset to pending
        // ACTUALLY, this is already handled elsewhere, maybe not...
        if ( stepKEY !== SC.STEP_KEY_VA) {  // VA ignores pending status
            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-status").setText(oBundle.getText("STEP.STATUS.PENDING"));
        } else {
            // saspfj, ToDo, determine if VA needs to handle this situation
        }

        // and the timer should be wiped out
        //sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-timer").setText("");
        //$("#FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-timer").textContent = "";
        $("#FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-timer").html("");

        // back to regular image
        sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "-image").setSrc("html/resources/sas/ea/images/" + stepKEY + ".png");

        // now reset the node
        // saspfj, ToDo, VERN, determine if this code really needs to be here
        var ctrl = sap.ui.getCore().byId("FLOW" + uidUi + "NODE" + stepKEY);
        if (ctrl) {
            ctrl.setIcon("html/resources/sas/ea/images/" + stepKEY + SC.STEP_STATUS_PENDING + "Node.png");
        }
    },

    clearStepArtifactsECLVPARA: function ( uidUi, stepKEY ) {
        //debugger;
        if ( sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsR") ) {
            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsR").destroyContent();
        }
        if (sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsD")){
            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsD").destroyContent();
        }
        if ( sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsO") ) {
            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + stepKEY + "OutsO").destroyContent();
        }
    },

    clearStepArtifactsVA: function ( uidUi, stepKEY ) {

        // remove all of them from UI
        if ( sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_VA + "OutsA") ) {
            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_VA + "OutsA").destroyContent();
        }
        if (sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_VA + "OutsV")){
            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_VA + "OutsV").destroyContent();
        }
        if ( sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_VA + "OutsO") ) {
            sap.ui.getCore().byId("FLOW" + uidUi + "STEPPANEL_" + SC.STEP_KEY_VA + "OutsO").destroyContent();
        }

        // saspfj, NOTE!!!, make sure this code is similar to code in Step.updateStepReportsAvailable()

        // if this is null, then we don't have an analysis job, so nothing else to do
        //if ( sas.ea.util.Step.dlgLoadLASRoptions.jsonParms === null ) {
        var oVAparms;
        if ( (oVAparms = sas.ea.util.JobResults.getVAparmsForUID( uidUi )) === null ) {
            return;
        }

        // update which aren't available anymore & all downstream ones
        //noinspection FallThroughInSwitchStatementJS
        switch ( stepKEY ) {
            case SC.STEP_KEY_EC:
                // none for EC :-(

            case SC.STEP_KEY_LV:
                // none for LV :-(

            case SC.STEP_KEY_PA:
                SEA.log( "VAparms", "2,3&4 available:false" );
                oVAparms.reports[2].bAvail = false;
                oVAparms.reports[3].bAvail = false;
                oVAparms.reports[4].bAvail = false;

            case SC.STEP_KEY_RA:
                SEA.log( "VAparms", "5&6 available:false" );
                oVAparms.reports[5].bAvail = false;
                oVAparms.reports[6].bAvail = false;
                break;
        }
    },

    clearAllPendingArtifacts: function ( jobID ) {
        //debugger;
        SEA.log( "Render", "clearAllPendingArtifacts: " + jobID );

        var uidUi = sas.ea.util.JobResults.getUiUID( jobID );

        // search for this job
        var aJobs = SEA.aRp[SEA.iCurrentTab].oJobs.jobs;
        for ( var x=0; x < aJobs.length; ++x ) {
            // if we find it
            if ( aJobs[x].jobID === jobID ) {
                var aSteps = aJobs[x].steps;
                // go through each step
                for ( var y=0; y < aSteps.length; ++y ) {
                    if ( aSteps[y].status === SC.STEP_STATUS_PENDING ) {
                        sas.ea.util.JobResults.clearStepArtifacts( uidUi, aSteps[y].progID );
                    }
                }
            }
        }
    },

    placeholderForCommas: function () {}
};
