/*****************************************************************************
 * 
 *  Program:     sea_autoexec.sas
 * 
 *  Author:      Chris Diering
 * 
 *  Date:        February, 2014
 * 
 *  Purpose:     Autoexec for SAS Episode Analysis.  Creates the global macro 
 *               variables and other global settings. 
 * 
 *  Parameters:  N/A
 * 
 *  V1 Replace:  N/A
 *
 *  Macro Calls: N/A
 * 
 *  HISTORY     | ID      | DATE        | COMMENT
 *              | chdier  | 04/07/2014  | Initial Creation
 *              | chdier  | 05/14/2014  | DE4344
 *              | chdier  | 07/10/2014  | US7942
 *              | chdier  | 08/06/2014	| US8020
 *              | sej     | 08/11/2014	| US8152 rename defdata to metadata
 *              | sej     | 06/18/2015	| Remove unused 2.2 code
 * 
 *  Notes:       
 *
 *******************************************************************************/

/*==============================================================================
Copyright (c) 2015 by SAS Institute Inc., Cary, NC 27513 USA
- All rights reserved
********************************************************************************/

%let sashome=%SYSGET(SASROOT);
%put &sashome;

%let ucmacro_dir=&sashome/ucmacros/avaecrsvr;
%put &ucmacro_dir;

options compress=char;

options append=(sasautos=&ucmacro_dir);

%Global SEA_SAVE_LOG /*Macro variable that indicates if the logs should be saved to a file */
	SEA_COMPRESS_OPTION /*Macro variable of if the compress option should be turned on */
        SEA_DLM /*Macro variable that contains the delimiter of the directories of the system */	
	;

/*Set the flag that determines if the log file gets redirected to an external file */
%let SEA_SAVE_LOG= Y;

%let SEA_COMPRESS_OPTION = Y;

/*determine the system SEA is running on to set the SEA_DLM macro variable */
data _null_;
	if "&SYSSCP" = 'WIN' then
		do; /*if the system that SAS Episode Analytics is running on is Windows set the SEA_DLM to '/' */
			call symputx('SEA_DLM','\','G');
		end; /*if the system that SAS Episode Analytics is running on is Windows set the SEA_DLM to '/' */
	else
		do;/*if the system that SAS Episode Analytics is running on is Unix set the SEA_DLM to '/' */
			call symputx('SEA_DLM','/','G');
		end;/*if the system that SAS Episode Analytics is running on is Unix set the SEA_DLM to '/' */
run;

/*****************************************************************************************/
options notes fullstimer msglevel=i;
options minoperator missing=' '; /* For Provider Attribution */
options spool; /* helps with macro debugging */
options ls=160 nosyntaxcheck sortsize=32G nodate;
options source source2 mprint mlogic mtrace symbolgen;

/*****************************************************************************************/
