/*
---------------------------------------------------------------------------
$Revision: 1.1.2.1 $
$Date: 2017/10/05 14:59:43 $

NAME:
sea_create_haf_audit

TYPE:
compiled macro

DESCRIPTION:
Creates the haf_audit table when a SEA 3.1 environment already exists

SAS VERSION:
9.3

INPUT:
Specify data sets, files, reports, graphs, html, e-mail, pages,
macro variables, slists, return codes, etc.


OUTPUT:
Specify data sets, files, reports, graphs, html, e-mail, pages,
macro variables, slists, return codes, etc.

+-----------------------------------------------------------------
| Output Item Name: macro vars containing table row counts
|   Description: macro variables containing the row count for
|                non-intersection tables
|   Type:  global macro variable
|   Mode: Optional
+-----------------------------------------------------------------
| Output Item Name: haf_audit
|   Description:  Oracle table that contains oracle db auditing
|   Type: data set
|   Mode: Always created
+-----------------------------------------------------------------

CALLS:
Specify external software units and how are they called (i.e.
by macros, methods, frames, etc.).

%sea_etl_macro_vars;
%sea_utl_current_os
%sea_utl_set_datamodel_location
%sea_ora_connect_info
%sea_error

CALLED BY (optional):


PRECONDITIONS (optional): %sea_ora_connect_info needs to be called prior to
calling this macro

POSTCONDITIONS (optional):

HISTORY:
userid   date       defectid  desc
chdier  20170913    S1369815  initial coding
------------------------------------------------------------------------------
Copyright (c) 2005-2012, SAS Institute Inc., Cary, NC, USA, All Rights Reserved
------------------------------------------------------------------------------
*/
%macro sea_create_haf_audit;
	%global retcode;
	%let retcode=0;
	%let errmsg=;
	%let jobdone=0;
	%let stamp=;

	data _null_;
		call symput('start',datetime());
	run;

	/* set global macro variables */
	%sea_etl_macro_vars;

	/* Verify that SEA is licensed */
	%if &basic_product_licensed = N %then
		%do;
			%let retcode=1;
			%let errmsg=%sysfunc(sasmsg(&msg_dset,_not_licensed_error,noquote));
			%GOTO EXIT;
		%end;

	%sea_utl_current_os;

	%if %bquote(&separator) = %str() or %bquote(&separator) = %str(UNK) %then
		%do;
			%let retcode = 1;
			%let errmsg=%sysfunc(sasmsg(&msg_dset
				,_etl_os_not_determined
				,noquote
				));
			%GOTO EXIT;
		%end;

	/* Get location of DDL and ATTRIB files */
	%sea_utl_set_datamodel_location;

	%if &retcode=1 %then
		%do;
			%goto EXIT;
		%end;

	%sea_ora_connect_info(lib_ext=&sea_ora_lib_ext
		,reset_lib=Y
		,outdsn=oracle_libraries);

	%if &retcode=1 %then
		%do;
			%GOTO EXIT;
		%end;

	/* Error out oracle libref not assigned */
	%if (%sysfunc(libref(&oralib))) > 0 %then
		%do;
			%let retcode = 1;
			%let errmsg=%sysfunc(sasmsg(&msg_dset
				,_seap01_error
				,noquote
				,&schema
				));
			%GOTO EXIT;
		%end;

	/* assign ddl macro location and add to sasautos */
	options source2;

	%IF (&SCHEMA > "" AND %INDEX(&SCHEMA, ".") = 0) %then
		%LET SCHEMA=%str(&SCHEMA).;

	%if %sysfunc(exist(&oralib..haf_audit)) eq 0 %then
		%do;

			proc sql;
				&ORA_PASS_THRU_CONNECT;

				%include "&sea_oracle_ddl_loc.&separator.haf_audit.sas";
			quit;

		%end;

	%if %sysfunc(exist(&oralib..haf_audit)) eq 0 %then
		%do;
			%let retcode = 1;
			%let errmsg=%sysfunc(sasmsg(&msg_dset
				,_file_notexist_error
				,noquote
				,haf_audit
				));
			%GOTO EXIT;
		%end;

%EXIT:

	%if &retcode=1 %then
		%do;
			%sea_error(msg1=&errmsg);
			%return;
		%end;
%mend;
